@@ -1,8 +1,13 @@
+from __future__ import annotations
+
 # standard libraries
+import asyncio
 import copy
 import datetime
 import gettext
 import numpy
+import numpy.typing
+import pathlib
 import threading
 import time
 import typing
@@ -16,27 +21,43 @@ from nion.utils import Event
 from nion.utils import Geometry
 from nion.utils import Registry
 
-from . import InstrumentDevice
-
 # other plug-ins
 from nion.instrumentation import camera_base
+from . import RonchigramCameraSimulator
+from . import EELSCameraSimulator
+
+if typing.TYPE_CHECKING:
+    from . import InstrumentDevice
+    from . import CameraSimulator
+    from . import ScanDevice
+
+_NDArray = numpy.typing.NDArray[typing.Any]
 
 _ = gettext.gettext
 
 
-class Camera(camera_base.CameraDevice):
+class Camera(camera_base.CameraDevice3):
     """Implement a camera device."""
 
     def __init__(self, camera_id: str, camera_type: str, camera_name: str, instrument: InstrumentDevice.Instrument):
         self.camera_id = camera_id
         self.camera_type = camera_type
         self.camera_name = camera_name
+        self.__camera_task: typing.Optional[CameraTask] = None
         self.__instrument = instrument
+        self.__scan_device: typing.Optional[ScanDevice.Device] = None
+        self.__simulator: CameraSimulator.CameraSimulator
+        if camera_type == "ronchigram":
+            self.__simulator = RonchigramCameraSimulator.RonchigramCameraSimulator(instrument, Geometry.IntSize.make(instrument.camera_sensor_dimensions("ronchigram")), instrument.counts_per_electron, instrument.stage_size_nm)
+        elif camera_type == "eels":
+            self.__simulator = EELSCameraSimulator.EELSCameraSimulator(instrument, Geometry.IntSize.make(instrument.camera_sensor_dimensions("eels")), instrument.counts_per_electron)
+        else:
+            raise ValueError(f"Unsupported camera type '{camera_type}'.")
         self.__sensor_dimensions = instrument.camera_sensor_dimensions(camera_type)
         self.__readout_area = instrument.camera_readout_area(camera_type)
         self.__symmetric_binning = True
         self.__integration_count = 1
-        self.__xdata_buffer = None
+        self.__xdata_buffer: typing.Optional[DataAndMetadata.DataAndMetadata] = None
         self.__frame_number = 0
         self.__thread = threading.Thread(target=self.__acquisition_thread)
         self.__thread_event = threading.Event()
@@ -48,58 +69,48 @@ class Camera(camera_base.CameraDevice):
         self.__cancel_sequence_event = threading.Event()
         self.__exposure = 1.0
         self.__binning = 1
-        self.__processing = None
+        self.__processing: typing.Optional[str] = None
+        self.__mask_array: typing.Optional[_NDArray] = None
         self.__thread.start()
 
-    def close(self):
+        # Also register the camera device and use a unique name for it so that we can directly access it
+        if Registry.get_component(f"usim_{camera_type}_camera_device"):
+            raise RuntimeError(f"Component 'usim_{camera_type}_camera_device' is already registered.")
+        Registry.register_component(self, {f"usim_{camera_type}_camera_device"})
+
+        # TODO Define external trigger interface and the mechanism for aqcuiring multiple sequences
+        self._external_trigger = False # Just here for testing, we need to decide on how to specify external trigger mode in the base class
+
+    def close(self) -> None:
         self.__cancel = True
         self.__thread_event.set()
         self.__thread.join()
-        self.__thread = None
-
-    def _test_rapid_changes(self, queue_task):
-        delay = 0.05
-        def set_mode(m):
-            self.mode = m
-        def t():
-            import functools
-            for i in range(1):
-                queue_task(functools.partial(set_mode, "run"))
-                # queue_task(functools.partial(set_binning, 1))
-                # set_exposure(0.22, 3)
-                time.sleep(delay)
-                queue_task(functools.partial(set_mode, "snap"))
-                # queue_task(functools.partial(set_binning, 2))
-                # set_exposure(0.33, 3)
-                time.sleep(delay)
-                queue_task(functools.partial(set_mode, "tune"))
-                # queue_task(functools.partial(set_binning, 4))
-                # set_exposure(0.44, 3)
-                time.sleep(delay)
-        threading.Thread(target=t).start()
+        self.__thread = typing.cast(typing.Any, None)
+        self.__simulator.close()
+        Registry.unregister_component(self)
 
     @property
-    def sensor_dimensions(self) -> (int, int):
+    def sensor_dimensions(self) -> typing.Tuple[int, int]:
         """Return the maximum sensor dimensions."""
         return self.__sensor_dimensions
 
     @property
-    def readout_area(self) -> (int, int, int, int):
+    def readout_area(self) -> typing.Tuple[int, int, int, int]:
         """Return the readout area TLBR, returned in sensor coordinates (unbinned)."""
         return self.__readout_area
 
     @readout_area.setter
-    def readout_area(self, readout_area_TLBR: (int, int, int, int)) -> None:
+    def readout_area(self, readout_area_TLBR: typing.Tuple[int, int, int, int]) -> None:
         """Set the readout area, specified in sensor coordinates (unbinned). Affects all modes."""
         self.__readout_area = readout_area_TLBR
 
     @property
-    def flip(self):
+    def flip(self) -> bool:
         """Return whether data is flipped left-right (last dimension)."""
         return False
 
     @flip.setter
-    def flip(self, do_flip):
+    def flip(self, do_flip: bool) -> None:
         """Set whether data is flipped left-right (last dimension). Affects all modes."""
         pass
 
@@ -108,19 +119,32 @@ class Camera(camera_base.CameraDevice):
         """Return possible binning values."""
         return [1, 2, 4, 8]
 
-    def get_expected_dimensions(self, binning: int) -> (int, int):
+    @property
+    def mask_array(self) -> typing.Optional[_NDArray]:
+        return self.__mask_array
+
+    @property
+    def simulator(self) -> CameraSimulator.CameraSimulator:
+        return self.__simulator
+
+    def get_expected_dimensions(self, binning: int) -> typing.Tuple[int, int]:
         """Return expected dimensions for the given binning value."""
         readout_area = self.__readout_area
         return (readout_area[2] - readout_area[0]) // binning, (readout_area[3] - readout_area[1]) // binning
 
-    def set_frame_parameters(self, frame_parameters) -> None:
+    def set_frame_parameters(self, frame_parameters: camera_base.CameraFrameParameters) -> None:
+        self.__set_frame_parameters(frame_parameters)
+
+    def __set_frame_parameters(self, frame_parameters: camera_base.CameraFrameParameters) -> None:
         self.__exposure = frame_parameters.exposure_ms / 1000
         self.__binning = frame_parameters.binning
         self.__processing = frame_parameters.processing
         self.__integration_count = frame_parameters.integration_count
+        mask_array = [mask.get_mask_array(self.get_expected_dimensions(self.__binning)) for mask in frame_parameters.active_masks]
+        self.__mask_array = numpy.array(mask_array) if mask_array else None
 
     @property
-    def calibration_controls(self) -> dict:
+    def calibration_controls(self) -> typing.Mapping[str, typing.Union[str, int, float]]:
         """Define the STEM calibration controls for this camera.
 
         The controls should be unique for each camera if there are more than one.
@@ -136,6 +160,9 @@ class Camera(camera_base.CameraDevice):
             "counts_per_electron_value": self.__instrument.counts_per_electron
         }
 
+    def get_dimensional_calibrations(self, readout_area: typing.Optional[Geometry.IntRect], binning_shape: typing.Optional[Geometry.IntSize]) -> typing.Sequence[Calibration.Calibration]:
+        return self.__simulator.get_dimensional_calibrations(readout_area, binning_shape)
+
     def start_live(self) -> None:
         """Start live acquisition. Required before using acquire_image."""
         if not self.__is_playing:
@@ -150,23 +177,27 @@ class Camera(camera_base.CameraDevice):
         self.__thread_event.set()
         # has_data_event is cleared in the acquisition loop after stopping
 
-    def acquire_image(self) -> dict:
+    def acquire_image(self) -> ImportExportManager.DataElementType:
         return self.__acquire_image(direct=False)
 
-    def __acquire_image(self, *, direct: bool) -> dict:
-        """Acquire the most recent data."""
+    def __acquire_image(self, *, direct: bool, index: int = 0) -> ImportExportManager.DataElementType:
+        """Acquire the most recent data.
+
+        The direct parameter is a shortcut to optimize the speed during sequence acquisition. When
+        set to True, it uses a direct form of acquisition that does not sync with the data thread.
+
+        The index parameter is used if direct is True; it allows direct to synchronize with the
+        start of the acquistion thread.
+        """
         xdata_buffer = None
         integration_count = self.__integration_count or 1
         for frame_number in range(integration_count):
             if direct:
-                self.__instrument.wait_for_camera_ack(self.__cancel_sequence_event)
-                if self.__direct_acquire(self.__cancel_sequence_event):
-                    self.__instrument.trigger_camera_frame()
-                self.__has_data_event.clear()
+                self.__direct_acquire(self.__cancel_sequence_event, index == 0)
             else:
                 if not self.__has_data_event.wait(self.__exposure * 200) and not self.__thread.is_alive():
                     raise Exception("No simulator thread.")
-                self.__has_data_event.clear()
+            self.__has_data_event.clear()
             if xdata_buffer is None:
                 xdata_buffer = copy.deepcopy(self.__xdata_buffer)
             else:
@@ -175,8 +206,9 @@ class Camera(camera_base.CameraDevice):
         # note: the data element will include spatial calibrations; but the camera adapter won't use them
         # right now (future fix); it uses a call to 'calibrations' instead.
         # whatever is in "hardware_source" will go into "properties" of data element
+        assert xdata_buffer
         assert len(xdata_buffer.dimensional_shape) == 2
-        data_element = dict()
+        data_element: typing.Dict[str, typing.Any] = dict()
         data_element["version"] = 1
         data_element["data"] = xdata_buffer.data
         data_element["timestamp"] = xdata_buffer.timestamp
@@ -185,30 +217,34 @@ class Camera(camera_base.CameraDevice):
         data_element["properties"]["integration_count"] = integration_count
         # data that has been binned vertically to a single row will be converted to 1D
         if xdata_buffer.dimensional_shape[0] == 1:
-            data_element["data"] = numpy.squeeze(xdata_buffer.data)
+            data_element["data"] = numpy.squeeze(xdata_buffer._data_ex)
             data_element["collection_dimension_count"] = 0
             data_element["datum_dimension_count"] = 1
         return data_element
 
-    def acquire_sequence_prepare(self, n: int) -> None:
-        self.__cancel_sequence_event.clear()
-
-    def acquire_sequence(self, n: int) -> typing.Optional[typing.Dict]:
+    def _acquire_sequence(self, n: int) -> typing.Optional[ImportExportManager.DataElementType]:
         # if the device does not implement acquire_sequence, fall back to looping acquisition.
         self.__is_acquiring = True
         self.__has_data_event.clear()  # ensure any has_data_event is new data
-        self.__instrument.sequence_progress = 0
         try:
             properties = None
             data = None
+            if self._external_trigger:
+                scan_device: typing.Optional[ScanDevice.Device] = Registry.get_component("scan_device")
+                if scan_device and hasattr(scan_device, "blanker_signal_condition"):
+                    with scan_device.blanker_signal_condition:
+                        scan_device.blanker_signal_condition.wait(timeout=max(self.__exposure * 2, 5))
             for index in range(n):
                 if self.__cancel_sequence_event.is_set():
                     return None
-                frame_data_element = self.__acquire_image(direct=True)
+                frame_data_element = self.__acquire_image(direct=True, index=index)
                 frame_data = frame_data_element["data"]
                 if self.__processing == "sum_project" and len(frame_data.shape) > 1:
                     data_shape = (n,) + frame_data.shape[1:]
                     data_dtype = frame_data.dtype
+                elif self.__processing == "sum_masked":
+                    data_shape = (n, len(self.__mask_array) if self.__mask_array is not None else 1)
+                    data_dtype = frame_data.dtype
                 else:
                     data_shape = (n,) + frame_data.shape
                     data_dtype = frame_data.dtype
@@ -217,7 +253,20 @@ class Camera(camera_base.CameraDevice):
                 assert data.shape == data_shape
                 assert data.dtype == data_dtype
                 if self.__processing == "sum_project" and len(frame_data.shape) > 1:
-                    data[index] = Core.function_sum(DataAndMetadata.new_data_and_metadata(frame_data), 0).data
+                    summed_xdata = Core.function_sum(DataAndMetadata.new_data_and_metadata(frame_data), 0)
+                    assert summed_xdata
+                    summed_data = summed_xdata.data
+                    assert summed_data is not None
+                    data[index] = summed_data
+                elif self.__processing == "sum_masked":
+                    if self.__mask_array is not None:
+                        summed_xdata = Core.function_sum(DataAndMetadata.new_data_and_metadata(frame_data * self.__mask_array), (1, 2))
+                        assert summed_xdata
+                        summed_data = summed_xdata.data
+                        assert summed_data is not None
+                        data[index] = summed_data
+                    else:
+                        data[index] = numpy.sum(frame_data)
                 else:
                     data[index] = frame_data
                 properties = copy.deepcopy(frame_data_element["properties"])
@@ -226,7 +275,6 @@ class Camera(camera_base.CameraDevice):
                     spatial_properties = properties.get("spatial_calibrations")
                     if spatial_properties is not None:
                         properties["spatial_calibrations"] = spatial_properties[1:]
-                self.__instrument.increment_sequence_progress()
         finally:
             self.__is_acquiring = False
         data_element = dict()
@@ -237,11 +285,32 @@ class Camera(camera_base.CameraDevice):
     def acquire_sequence_cancel(self) -> None:
         self.__cancel_sequence_event.set()
 
-    def __direct_acquire(self, cancel_event):
+    def acquire_sequence_begin(self, camera_frame_parameters: camera_base.CameraFrameParameters, count: int, **kwargs: typing.Any) -> camera_base.PartialData:
+        self.__cancel_sequence_event.clear()
+        self.__set_frame_parameters(camera_frame_parameters)
+        self.__camera_task = CameraTask(self, camera_frame_parameters, (count,))
+        self.__camera_task.start()
+        return camera_base.PartialData(self.__camera_task._xdata_ex, False, False, None, 0)
+
+    def acquire_sequence_continue(self, *, update_period: float = 1.0, **kwargs: typing.Any) -> camera_base.PartialData:
+        assert self.__camera_task
+        is_complete, is_canceled, valid_count = self.__camera_task.grab_partial(update_period=update_period)
+        return camera_base.PartialData(self.__camera_task._xdata_ex, is_complete, is_canceled, None, valid_count)
+
+    def acquire_sequence_end(self, **kwargs: typing.Any) -> None:
+        self.__camera_task = None
+
+    def __direct_acquire(self, cancel_event: threading.Event, do_sync: bool = False) -> bool:
+        # if do_sync is True, the scan device will sync with its acquisition thread before
+        # advancing the pixel. this avoids race conditions of when the pixel count gets reset.
+        # this will be set only for the first frame in a sequence.
         start = time.time()
         readout_area = self.readout_area
         binning_shape = Geometry.IntSize(self.__binning, self.__binning if self.__symmetric_binning else 1)
-        xdata = self.__instrument.get_camera_data(self.camera_type, Geometry.IntRect.from_tlbr(*readout_area), binning_shape, self.__exposure)
+        # scan device is only set during synchronized acquisition
+        if self.__scan_device:
+            self.__scan_device.advance_pixel(do_sync=do_sync)
+        xdata = self.__simulator.get_frame_data(Geometry.IntRect.from_tlbr(*readout_area), binning_shape, self.__exposure, self.__instrument.scan_context, self.__instrument.probe_position)
         self.__acquired_one_event.set()
         elapsed = time.time() - start
         wait_s = max(self.__exposure - elapsed, 0)
@@ -252,7 +321,7 @@ class Camera(camera_base.CameraDevice):
             return True
         return False
 
-    def __acquisition_thread(self):
+    def __acquisition_thread(self) -> None:
         while True:
             if self.__cancel:  # case where exposure was canceled.
                 break
@@ -268,98 +337,95 @@ class Camera(camera_base.CameraDevice):
                     self.__has_data_event.clear()
                     self.__thread_event.clear()
 
-    PartialData = camera_base.CameraHardwareSource.PartialData
+    def acquire_synchronized_begin(self, camera_frame_parameters: camera_base.CameraFrameParameters, collection_shape: DataAndMetadata.ShapeType, **kwargs: typing.Any) -> camera_base.PartialData:
+        self.__cancel_sequence_event.clear()
+        self.__set_frame_parameters(camera_frame_parameters)
+        self.__scan_device = Registry.get_component("scan_device")
+        self.__camera_task = CameraTask(self, camera_frame_parameters, collection_shape)
+        self.__camera_task.start()
+        return camera_base.PartialData(self.__camera_task._xdata_ex, False, False, None, 0)
 
-    class CameraTask:
-        def __init__(self, camera_device: "Camera", camera_frame_parameters, scan_shape: typing.Tuple[int, ...]):
-            self.__camera_device = camera_device
-            self.__camera_frame_parameters = camera_frame_parameters
-            self.__scan_shape = scan_shape
-            self.__scan_count = int(numpy.product(self.__scan_shape))
-            self.__aborted = False
-            self.__data = None
-            self.__xdata = None
-            self.__start = 0
+    def acquire_synchronized_continue(self, *, update_period: float = 1.0, **kwargs: typing.Any) -> camera_base.PartialData:
+        assert self.__camera_task
+        is_complete, is_canceled, valid_count = self.__camera_task.grab_partial(update_period=update_period)
+        return camera_base.PartialData(self.__camera_task._xdata_ex, is_complete, is_canceled, None, valid_count)
 
-        @property
-        def xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
-            return self.__xdata
+    def acquire_synchronized_end(self, **kwargs: typing.Any) -> None:
+        self.__camera_task = None
+        self.__scan_device = None
 
-        def start(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
-            # returns the full scan readout, including flyback pixels
-            camera_readout_shape = self.__camera_device.get_expected_dimensions(self.__camera_frame_parameters.get("binning", 1))
-            if self.__camera_frame_parameters.get("processing") == "sum_project":
-                camera_readout_shape = camera_readout_shape[1:]
-            self.__data = numpy.zeros(self.__scan_shape + camera_readout_shape, numpy.float32)
-            data_descriptor = DataAndMetadata.DataDescriptor(False, len(self.__scan_shape), len(camera_readout_shape))
-            self.__xdata = DataAndMetadata.new_data_and_metadata(self.__data, data_descriptor=data_descriptor)
-            return self.__xdata
+    def acquire_synchronized_cancel(self) -> None:
+        self.__cancel_sequence_event.set()
 
-        def grab_partial(self, *, update_period: float = 1.0) -> typing.Tuple[bool, bool, int]:
-            # updates the full scan readout data, returns a tuple of is complete, is canceled, and
-            # the number of valid rows.
-            start = self.__start
-            frame_parameters = self.__camera_frame_parameters
-            exposure = frame_parameters.exposure_ms / 1000.0
-            row_size = self.__scan_shape[-1]
-            n = min(max((int(update_period / exposure) + row_size - 1) // row_size * row_size, row_size), self.__scan_count - start)
-            is_complete = start + n == self.__scan_count
-            # print(f"{start=} {n=} {self.__scan_count=} {is_complete=}")
-            data_element = self.__camera_device.acquire_sequence((n))
-            if data_element and not self.__aborted:
-                xdata = ImportExportManager.convert_data_element_to_data_and_metadata(data_element)
+    @property
+    def _is_acquire_synchronized_running(self) -> bool:
+        return self.__camera_task is not None
+
+
+class CameraTask:
+    def __init__(self, camera_device: Camera, camera_frame_parameters: camera_base.CameraFrameParameters, collection_shape: typing.Tuple[int, ...]) -> None:
+        self.__camera_device = camera_device
+        self.__camera_frame_parameters = camera_frame_parameters
+        self.__collection_shape = collection_shape
+        self.__count = int(numpy.product(self.__collection_shape))
+        self.__aborted = False
+        self.__data: typing.Optional[_NDArray] = None
+        self.__xdata: typing.Optional[DataAndMetadata.DataAndMetadata] = None
+        self.__start = 0
+
+    @property
+    def xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        return self.__xdata
+
+    @property
+    def _xdata_ex(self) -> DataAndMetadata.DataAndMetadata:
+        assert self.__xdata
+        return self.__xdata
+
+    def start(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        # returns the full readout, including flyback pixels
+        camera_readout_shape: typing.Tuple[int, ...] = self.__camera_device.get_expected_dimensions(self.__camera_frame_parameters.binning)
+        if self.__camera_frame_parameters.processing == "sum_project":
+            camera_readout_shape = camera_readout_shape[1:]
+        elif self.__camera_frame_parameters.processing == "sum_masked":
+            camera_readout_shape = (len(self.__camera_device.mask_array) if self.__camera_device.mask_array is not None else 1,)
+        self.__data = numpy.zeros(self.__collection_shape + camera_readout_shape, numpy.float32)
+        data_descriptor = DataAndMetadata.DataDescriptor(False, len(self.__collection_shape), len(camera_readout_shape))
+        self.__xdata = DataAndMetadata.new_data_and_metadata(self.__data, data_descriptor=data_descriptor)
+        return self.__xdata
+
+    def grab_partial(self, *, update_period: float = 1.0) -> typing.Tuple[bool, bool, int]:
+        # updates the full readout data, returns a tuple of is complete, is canceled, and
+        # the number of valid rows.
+        start = self.__start
+        frame_parameters = self.__camera_frame_parameters
+        exposure = frame_parameters.exposure_ms / 1000.0
+        n = min(max(int(update_period / exposure), 1), self.__count - start)
+        is_complete = start + n == self.__count
+        # print(f"{start=} {n=} {self.__count=} {is_complete=}")
+        data_element = self.__camera_device._acquire_sequence(n)
+        if data_element and not self.__aborted:
+            xdata = ImportExportManager.convert_data_element_to_data_and_metadata(data_element)
+            dimensional_calibrations = tuple(Calibration.Calibration() for _ in range(len(self.__collection_shape))) + tuple(xdata.dimensional_calibrations[1:])
+            assert self.__xdata
+            self.__xdata._set_intensity_calibration(xdata.intensity_calibration)
+            self.__xdata._set_dimensional_calibrations(dimensional_calibrations)
+            if len(self.__collection_shape) > 1:
+                row_size = self.__collection_shape[-1]
                 start_row = start // row_size
                 rows = n // row_size
-                dimensional_calibrations = tuple(Calibration.Calibration() for _ in range(len(self.__scan_shape))) + tuple(xdata.dimensional_calibrations[1:])
-                metadata = xdata.metadata
+                metadata = dict(xdata.metadata)
                 metadata.setdefault("hardware_source", dict())["valid_rows"] = start_row + rows
-                self.__xdata._set_intensity_calibration(xdata.intensity_calibration)
-                self.__xdata._set_dimensional_calibrations(dimensional_calibrations)
                 self.__xdata._set_metadata(metadata)
-                self.__data[start_row:start_row + rows, ...] = xdata.data.reshape((rows, row_size) + xdata.data.shape[1:])
-                self.__start = start + n
-                return is_complete, False, start_row + rows
-            self.__start = 0
-            return True, True, 0
-
-    def acquire_synchronized_begin(self, camera_frame_parameters: typing.Mapping, scan_shape: typing.Tuple[int, ...], **kwargs) -> PartialData:
-        self.__camera_task = Camera.CameraTask(self, camera_frame_parameters, scan_shape)
-        self.__camera_task.start()
-        return Camera.PartialData(self.__camera_task.xdata, False, False, 0)
-
-    def acquire_synchronized_continue(self, *, update_period: float = 1.0, **kwargs) -> PartialData:
-        is_complete, is_canceled, valid_rows = self.__camera_task.grab_partial(update_period=update_period)
-        return Camera.PartialData(self.__camera_task.xdata, is_complete, is_canceled, valid_rows)
-
-    def acquire_synchronized_end(self) -> None:
-        self.__camera_task = None
-
-
-class CameraFrameParameters(dict):
-
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.__dict__ = self
-        self.exposure_ms = self.get("exposure_ms", 125)
-        self.binning = self.get("binning", 1)
-        self.processing = self.get("processing")
-        self.integration_count = self.get("integration_count")
-
-    def __copy__(self):
-        return self.__class__(copy.copy(dict(self)))
-
-    def __deepcopy__(self, memo):
-        deepcopy = self.__class__(copy.deepcopy(dict(self)))
-        memo[id(self)] = deepcopy
-        return deepcopy
-
-    def as_dict(self):
-        return {
-            "exposure_ms": self.exposure_ms,
-            "binning": self.binning,
-            "processing": self.processing,
-            "integration_count": self.integration_count,
-        }
+            else:
+                row_size = 1
+            # convert from a sequence to a collection.
+            assert self.__data is not None
+            self.__data.reshape((self.__data.shape[0] * row_size,) + self.__data.shape[len(self.__collection_shape):])[start:start + n, ...] = xdata._data_ex
+            self.__start = start + n
+            return is_complete, False, start + n
+        self.__start = 0
+        return True, True, 0
 
 
 class CameraSettings:
@@ -387,33 +453,37 @@ class CameraSettings:
 
         # configure profiles
         self.__settings = [
-            CameraFrameParameters({"exposure_ms": 100, "binning": 2}),
-            CameraFrameParameters({"exposure_ms": 200, "binning": 2}),
-            CameraFrameParameters({"exposure_ms": 500, "binning": 1}),
+            camera_base.CameraFrameParameters({"exposure_ms": 100, "binning": 2}),
+            camera_base.CameraFrameParameters({"exposure_ms": 200, "binning": 2}),
+            camera_base.CameraFrameParameters({"exposure_ms": 500, "binning": 1}),
         ]
 
         self.__current_settings_index = 0
-
+        self.__masks: typing.List[camera_base.Mask] = list()
         self.__frame_parameters = copy.deepcopy(self.__settings[self.__current_settings_index])
         self.__record_parameters = copy.deepcopy(self.__settings[-1])
 
-    def close(self):
+    def close(self) -> None:
         pass
 
-    def initialize(self, **kwargs):
+    def initialize(self, configuration_location: typing.Optional[pathlib.Path] = None, event_loop: typing.Optional[asyncio.AbstractEventLoop] = None, **kwargs: typing.Any) -> None:
         pass
 
-    def apply_settings(self, settings_dict: typing.Dict) -> None:
+    @property
+    def masks(self) -> typing.List[camera_base.Mask]:
+        return self.__masks
+
+    def apply_settings(self, settings_dict: typing.Mapping[str, typing.Any]) -> None:
         """Initialize the settings with the settings_dict."""
         if isinstance(settings_dict, dict):
             settings_list = settings_dict.get("settings", list())
             if len(settings_list) == 3:
-                self.__settings = [CameraFrameParameters(settings) for settings in settings_list]
+                self.__settings = [camera_base.CameraFrameParameters(settings) for settings in settings_list]
             self.__current_settings_index = settings_dict.get("current_settings_index", 0)
-            self.__frame_parameters = CameraFrameParameters(settings_dict.get("current_settings", self.__settings[0].as_dict()))
+            self.__frame_parameters = camera_base.CameraFrameParameters(settings_dict.get("current_settings", self.__settings[0].as_dict()))
             self.__record_parameters = copy.deepcopy(self.__settings[-1])
 
-    def __save_settings(self) -> typing.Dict:
+    def __save_settings(self) -> typing.Mapping[str, typing.Any]:
         settings_dict = {
             "settings": [settings.as_dict() for settings in self.__settings],
             "current_settings_index": self.__current_settings_index,
@@ -421,10 +491,10 @@ class CameraSettings:
         }
         return settings_dict
 
-    def get_frame_parameters_from_dict(self, d):
-        return CameraFrameParameters(d)
+    def get_frame_parameters_from_dict(self, d: typing.Mapping[str, typing.Any]) -> camera_base.CameraFrameParameters:
+        return camera_base.CameraFrameParameters(d)
 
-    def set_current_frame_parameters(self, frame_parameters: CameraFrameParameters) -> None:
+    def set_current_frame_parameters(self, frame_parameters: camera_base.CameraFrameParameters) -> None:
         """Set the current frame parameters.
 
         Fire the current frame parameters changed event and optionally the settings changed event.
@@ -433,11 +503,11 @@ class CameraSettings:
         self.settings_changed_event.fire(self.__save_settings())
         self.current_frame_parameters_changed_event.fire(frame_parameters)
 
-    def get_current_frame_parameters(self) -> CameraFrameParameters:
+    def get_current_frame_parameters(self) -> camera_base.CameraFrameParameters:
         """Get the current frame parameters."""
-        return CameraFrameParameters(self.__frame_parameters)
+        return camera_base.CameraFrameParameters(self.__frame_parameters.as_dict())
 
-    def set_record_frame_parameters(self, frame_parameters: CameraFrameParameters) -> None:
+    def set_record_frame_parameters(self, frame_parameters: camera_base.CameraFrameParameters) -> None:
         """Set the record frame parameters.
 
         Fire the record frame parameters changed event and optionally the settings changed event.
@@ -445,11 +515,11 @@ class CameraSettings:
         self.__record_parameters = copy.copy(frame_parameters)
         self.record_frame_parameters_changed_event.fire(frame_parameters)
 
-    def get_record_frame_parameters(self) -> CameraFrameParameters:
+    def get_record_frame_parameters(self) -> camera_base.CameraFrameParameters:
         """Get the record frame parameters."""
         return self.__record_parameters
 
-    def set_frame_parameters(self, settings_index: int, frame_parameters: CameraFrameParameters) -> None:
+    def set_frame_parameters(self, settings_index: int, frame_parameters: camera_base.CameraFrameParameters) -> None:
         """Set the frame parameters with the settings index and fire the frame parameters changed event.
 
         If the settings index matches the current settings index, call set current frame parameters.
@@ -467,7 +537,7 @@ class CameraSettings:
         self.settings_changed_event.fire(self.__save_settings())
         self.frame_parameters_changed_event.fire(settings_index, frame_parameters)
 
-    def get_frame_parameters(self, settings_index) -> CameraFrameParameters:
+    def get_frame_parameters(self, settings_index: int) -> camera_base.CameraFrameParameters:
         """Get the frame parameters for the settings index."""
         return copy.copy(self.__settings[settings_index])
 
@@ -511,17 +581,12 @@ class CameraModule:
 
 def run(instrument: InstrumentDevice.Instrument) -> None:
     component_types = {"camera_module"}  # the set of component types that this component represents
-
     camera_device = Camera("usim_ronchigram_camera", "ronchigram", _("uSim Ronchigram Camera"), instrument)
-    camera_device.camera_panel_type = "ronchigram"
-
+    setattr(camera_device, "camera_panel_type", "ronchigram")
     camera_settings = CameraSettings("usim_ronchigram_camera")
-
     Registry.register_component(CameraModule("usim_stem_controller", camera_device, camera_settings), component_types)
 
     camera_device = Camera("usim_eels_camera", "eels", _("uSim EELS Camera"), instrument)
-    camera_device.camera_panel_type = "eels"
-
+    setattr(camera_device, "camera_panel_type", "eels")
     camera_settings = CameraSettings("usim_eels_camera")
-
     Registry.register_component(CameraModule("usim_stem_controller", camera_device, camera_settings), component_types)
