@@ -1,9 +1,12 @@
+from __future__ import annotations
+
 # standard libraries
 import copy
 import typing
 import math
 import numpy
-import scipy.ndimage.interpolation
+import numpy.typing
+import scipy.ndimage
 import scipy.stats
 
 from nion.data import Calibration
@@ -14,6 +17,12 @@ from nion.utils import Geometry
 from . import CameraSimulator
 from . import Noise
 
+if typing.TYPE_CHECKING:
+    from . import InstrumentDevice
+    from nion.instrumentation import stem_controller
+
+_NDArray = numpy.typing.NDArray[typing.Any]
+
 
 class AberrationsController:
     """Track aberrations and apply them.
@@ -24,27 +33,24 @@ class AberrationsController:
     """
     coefficient_names = (
         "c0a", "c0b", "c10", "c12a", "c12b", "c21a", "c21b", "c23a", "c23b", "c30", "c32a", "c32b", "c34a", "c34b",
-        "c41a", "c41b", "c43a", "c43b" , "c45a", "c45b", "c50", "c52a", "c52b", "c54a", "c54b", "c56a", "c56b", "c70"
+        "c41a", "c41b", "c43a", "c43b", "c45a", "c45b", "c50", "c52a", "c52b", "c54a", "c54b", "c56a", "c56b", "c70"
     )
 
-    def __init__(self, height, width, theta, max_defocus, defocus):
+    def __init__(self, height: int, width: int, theta: float, max_defocus: float, defocus: float) -> None:
         self.__height = height
         self.__width = width
         self.__theta = theta
         self.__max_defocus = max_defocus
-        self.__coefficients = dict()
-        self.__intermediates = dict()  # functions of height/width/theta
-        self.__chis = dict()  # chi's, functions of intermediate and coefficients
+        self.__coefficients: typing.Dict[str, float] = dict()
+        self.__intermediates: typing.Dict[str, _NDArray] = dict()  # functions of height/width/theta
+        self.__chis: typing.Dict[str, _NDArray] = dict()  # chi's, functions of intermediate and coefficients
         self.__coefficients["c10"] = defocus
-        self.__chi = None
-        self.__c = None
+        self.__chi: typing.Optional[_NDArray] = None
+        self.__c: typing.Optional[typing.List[float]] = None
 
-    def apply(self, aberrations, data):
-        import time
-        t0 = time.perf_counter()
-
-        height = aberrations["height"]
-        width = aberrations["width"]
+    def apply(self, aberrations: typing.Mapping[str, typing.Union[int, float]], data: numpy.typing.NDArray[numpy.float32]) -> _NDArray:
+        height = int(aberrations["height"])
+        width = int(aberrations["width"])
         theta = aberrations["theta"]
 
         if theta != self.__theta or width != self.__width or height != self.__height:
@@ -66,7 +72,7 @@ class AberrationsController:
 
         # below: the tedious part...
 
-        def get_i0ab():
+        def get_i0ab() -> typing.Tuple[_NDArray, _NDArray]:
             i0a = self.__intermediates.get("c0a")
             i0b = self.__intermediates.get("c0b")
             if i0a is None or i0b is None:
@@ -75,41 +81,41 @@ class AberrationsController:
                 self.__intermediates["c0b"] = i0b
             return i0a, i0b
 
-        def get_i0a():
+        def get_i0a() -> _NDArray:
             return get_i0ab()[0]
 
-        def get_i0b():
+        def get_i0b() -> _NDArray:
             return get_i0ab()[1]
 
-        def get_i0a_squared():
+        def get_i0a_squared() -> _NDArray:
             i0a_squared = self.__intermediates.get("c0a_squared")
             if i0a_squared is None:
                 i0a_squared = get_i0a() ** 2
                 self.__intermediates["c0a_squared"] = i0a_squared
             return i0a_squared
 
-        def get_i0b_squared():
+        def get_i0b_squared() -> _NDArray:
             i0b_squared = self.__intermediates.get("c0b_squared")
             if i0b_squared is None:
                 i0b_squared = get_i0b() ** 2
                 self.__intermediates["c0b_squared"] = i0b_squared
             return i0b_squared
 
-        def get_iradius():
+        def get_iradius() -> _NDArray:
             ir = self.__intermediates.get("ir")
             if ir is None:
                 ir = get_i0a_squared() + get_i0b_squared()
                 self.__intermediates["ir"] = ir
             return ir
 
-        def get_idiff_sq():
+        def get_idiff_sq() -> _NDArray:
             ids = self.__intermediates.get("ids")
             if ids is None:
                 ids = get_i0a_squared() - get_i0b_squared()
                 self.__intermediates["ids"] = ids
             return ids
 
-        def get_intermediate(coefficient_name):
+        def get_intermediate(coefficient_name: str) -> _NDArray:
             intermediate = self.__intermediates.get(coefficient_name)
             if intermediate is None:
                 if coefficient_name == "c0a":
@@ -151,7 +157,8 @@ class AberrationsController:
                 elif coefficient_name == "c45a":
                     intermediate = (get_i0a() * get_idiff_sq() ** 2 - 4 * get_i0a() * get_idiff_sq() * get_i0b() ** 2 - 4 * get_i0a() ** 3 * get_i0b() ** 2) / 5
                 elif coefficient_name == "c45b":
-                    intermediate = (get_i0b() * get_idiff_sq() ** 2 + 4 * get_i0b() * get_idiff_sq() * get_i0a() ** 2 - 4 * get_i0a() ** 2 * get_i0b() ** 3) / 5
+                    # this type: ignore is inexplicably necessary; seems like bug in numpy.typing or mypy. try removing to see if it passes typing tests.
+                    intermediate = (get_i0b() * get_idiff_sq() ** 2 + 4 * get_i0b() * get_idiff_sq() * get_i0a() ** 2 - 4 * get_i0a() ** 2 * get_i0b() ** 3) / 5  # type: ignore
                 elif coefficient_name == "c50":
                     intermediate = 8 * get_intermediate("c10") ** 3 / 6
                 elif coefficient_name == "c52a":
@@ -168,9 +175,10 @@ class AberrationsController:
                     intermediate = get_intermediate("c12b") * get_idiff_sq() ** 2 - (4 * get_i0a() ** 3 * get_i0b() ** 3) / 3
                 elif coefficient_name == "c70":
                     intermediate = 2 * get_intermediate("c10") ** 4
+            assert intermediate is not None
             return intermediate
 
-        def get_chi(coefficient_name):
+        def get_chi(coefficient_name: str) -> typing.Optional[_NDArray]:
             chi = self.__chis.get(coefficient_name)
             if chi is None:
                 coefficient = self.__coefficients.get(coefficient_name, 0.0)
@@ -198,7 +206,7 @@ class AberrationsController:
         if self.__c is None and self.__chi is not None:
             # print("recalculating grad chi")
             grad_chi = numpy.gradient(self.__chi)
-            max_chi0 = self.__max_defocus * theta  * theta
+            max_chi0 = self.__max_defocus * theta * theta
             max_chi1 = self.__max_defocus * theta * theta * ((1 - 1 / width) * (1 - 1 / width) + (1 - 1 / height) * (1 - 1 / height)) / 2
             max_chi = max_chi0 - max_chi1
             scale_y = height / 2 / max_chi
@@ -209,23 +217,19 @@ class AberrationsController:
 
         if self.__c is not None:
             # scale the offsets so that at max defocus, the coordinates cover the entire area of data.
-            t1 = time.perf_counter()
-            r = scipy.ndimage.interpolation.map_coordinates(data, self.__c)
-            t2 = time.perf_counter()
-            # print(f"elapsed {t1 - t0} {t2 - t1}")
-            return r
+            return scipy.ndimage.map_coordinates(data, self.__c, order=1)  # type: ignore
 
         return numpy.zeros((height, width))
 
 
-def ellipse_radius(polar_angle: typing.Union[float, numpy.ndarray], a: float, b: float, rotation: float) -> typing.Union[float, numpy.ndarray]:
+def ellipse_radius(polar_angle: typing.Union[float, _NDArray], a: float, b: float, rotation: float) -> typing.Union[float, _NDArray]:
     """
     Returns the radius of a point lying on an ellipse with the given parameters. The ellipse is described in polar
     coordinates here, which makes it easy to incorporate a rotation.
 
     Parameters
     -----------
-    polar_angle : float or numpy.ndarray
+    polar_angle : float or _NDArray
                   Polar angle of a point to which the corresponding radius should be calculated (rad).
     a : float
         Length of the major half-axis of the ellipse.
@@ -235,15 +239,14 @@ def ellipse_radius(polar_angle: typing.Union[float, numpy.ndarray], a: float, b:
 
     Returns
     --------
-    radius : float or numpy.ndarray
+    radius : float or _NDArray
              Radius of a point lying on an ellipse with the given parameters.
     """
 
-    return a*b/numpy.sqrt((b*numpy.cos(polar_angle+rotation))**2+(a*numpy.sin(polar_angle+rotation))**2)
+    return a * b / numpy.sqrt((b * numpy.cos(polar_angle + rotation)) ** 2 + (a * numpy.sin(polar_angle + rotation)) ** 2)  # type: ignore
 
 
-def draw_ellipse(image: numpy.ndarray, ellipse: typing.Tuple[float, float, float, float, float], *,
-                 color: typing.Any=1.0) -> None:
+def draw_ellipse(image: _NDArray, ellipse: typing.Tuple[float, float, float, float, float], *, color: typing.Any = 1.0) -> None:
     """
     Draws an ellipse on a 2D-array.
 
@@ -268,49 +271,54 @@ def draw_ellipse(image: numpy.ndarray, ellipse: typing.Tuple[float, float, float
     """
     shape = image.shape
     assert len(shape) == 2, 'Can only draw an ellipse on a 2D-array.'
-    #coords = np.mgrid[-shape[0]/2:shape[0]/2:shape[0]*1j, -shape[1]/2:shape[1]/2:shape[1]*1j]
+    # coords = np.mgrid[-shape[0]/2:shape[0]/2:shape[0]*1j, -shape[1]/2:shape[1]/2:shape[1]*1j]
     top = max(int(ellipse[0] - ellipse[2]), 0)
     left = max(int(ellipse[1] - ellipse[2]), 0)
     bottom = min(int(ellipse[0] + ellipse[2]) + 1, shape[0])
     right = min(int(ellipse[1] + ellipse[2]) + 1, shape[1])
-    coords = numpy.mgrid[top-ellipse[0]:bottom-ellipse[0], left-ellipse[1]:right-ellipse[1]]
-    #coords[0] -= ellipse[0]
-    #coords[1] -= ellipse[1]
+    coords = numpy.mgrid[top - ellipse[0]:bottom - ellipse[0], left - ellipse[1]:right - ellipse[1]]  # type: ignore
+    # coords[0] -= ellipse[0]
+    # coords[1] -= ellipse[1]
     radii = numpy.sqrt(numpy.sum(coords**2, axis=0))
     polar_angles = numpy.arctan2(coords[0], coords[1])
     ellipse_radii = ellipse_radius(polar_angles, *ellipse[2:])
-    image[top:bottom, left:right][radii<ellipse_radii] = color
+    image[top:bottom, left:right][radii < ellipse_radii] = color
 
 
 class RonchigramCameraSimulator(CameraSimulator.CameraSimulator):
     depends_on = ["C10Control", "C12Control", "C21Control", "C23Control", "C30Control", "C32Control", "C34Control",
-                  "C34Control", "stage_position_m", "probe_state", "probe_position", "live_probe_position", "features",
+                  "C34Control", "stage_position_m", "probe_state", "probe_position", "features",
                   "beam_shift_m", "is_blanked", "BeamCurrent", "CAperture", "ApertureRound", "S_VOA", "ConvergenceAngle"]
 
-    def __init__(self, instrument, ronchigram_shape: Geometry.IntSize, counts_per_electron: int, stage_size_nm: float):
+    def __init__(self, instrument: InstrumentDevice.Instrument, ronchigram_shape: Geometry.IntSize, counts_per_electron: int, stage_size_nm: float) -> None:
         super().__init__(instrument, "ronchigram", ronchigram_shape, counts_per_electron)
-        self.__last_sample = None
-        self.__cached_frame = None
+        self.__cached_frame: typing.Optional[DataAndMetadata.DataAndMetadata] = None
         max_defocus = instrument.max_defocus
-        tv_pixel_angle = math.asin(instrument.stage_size_nm / (max_defocus * 1E9)) / ronchigram_shape.height
-        self.__tv_pixel_angle = tv_pixel_angle
         self.__stage_size_nm = stage_size_nm
-        self.__max_defocus = max_defocus
         self.__data_scale = 1.0
-        self.__aperture_ellipse = None
+        self.__aperture_ellipse: typing.Optional[typing.Tuple[float, float, float, float, float]] = None
         self.__aperture_mask = None
-        theta = tv_pixel_angle * ronchigram_shape.height / 2  # half angle on camera
+        theta = self._tv_pixel_angle * ronchigram_shape.height / 2  # half angle on camera
         defocus_m = instrument.defocus_m
         self.__aberrations_controller = AberrationsController(ronchigram_shape.height, ronchigram_shape.width, theta, max_defocus, defocus_m)
         self.noise = Noise.PoissonNoise()
 
-    def _draw_aperture(self, frame_data: numpy.ndarray, binning_shape: Geometry.IntSize, enlarge_by: float=0.0):
+    @property
+    def _tv_pixel_angle(self) -> float:
+        instrument = self.instrument
+        return math.asin(instrument.stage_size_nm / (instrument.max_defocus * 1E9)) / self._camera_shape.height
+
+    @property
+    def _max_defocus(self) -> float:
+        return self.instrument.max_defocus
+
+    def _draw_aperture(self, frame_data: _NDArray, binning_shape: Geometry.IntSize, enlarge_by: float = 0.0) -> None:
         # TODO handle asymmetric binning
         binning = binning_shape[0]
         position = self.instrument.GetVal2D("CAperture")
         aperture_round = self.instrument.GetVal2D("ApertureRound")
         shape = frame_data.shape
-        ellipse_center = 0.5*shape[0] + position.y / self.__tv_pixel_angle / binning, 0.5*shape[1] + position.x / self.__tv_pixel_angle / binning
+        ellipse_center = 0.5 * shape[0] + position.y / self._tv_pixel_angle / binning, 0.5 * shape[1] + position.x / self._tv_pixel_angle / binning
         excentricity = math.sqrt(aperture_round[0]**2 + aperture_round[1]**2)
         # adapt excentricity so that control behaves linearly and is defined for all values
         # this is the modified inverse function of the calculation of the major half-axis a
@@ -318,7 +326,7 @@ class RonchigramCameraSimulator(CameraSimulator.CameraSimulator):
         direction = numpy.arctan2(aperture_round[0], aperture_round[1])
         # Calculate a and b (the ellipse half-axes) from excentricity. Keep ellipse area constant
         convergence_angle = self.instrument.GetVal("ConvergenceAngle") * (1 + enlarge_by)
-        convergence_angle_pixels = convergence_angle / self.__tv_pixel_angle / binning
+        convergence_angle_pixels = convergence_angle / self._tv_pixel_angle / binning
         a = math.sqrt(convergence_angle_pixels**2 / math.sqrt(1 - excentricity**2))
         b = convergence_angle_pixels**2 / a
         self.__aperture_ellipse = ellipse_center + (a, b, direction)
@@ -326,25 +334,25 @@ class RonchigramCameraSimulator(CameraSimulator.CameraSimulator):
         draw_ellipse(aperture_mask, self.__aperture_ellipse)
         frame_data *= aperture_mask
 
-    def get_frame_data(self, readout_area: Geometry.IntRect, binning_shape: Geometry.IntSize, exposure_s: float, scan_context, parked_probe_position) -> DataAndMetadata.DataAndMetadata:
-        # check if one of the arguments has changed since last call
-        new_frame_settings = [readout_area, binning_shape, exposure_s, copy.deepcopy(scan_context)]
-        if new_frame_settings != self._last_frame_settings:
+    def get_frame_data(self, readout_area: Geometry.IntRect, binning_shape: Geometry.IntSize, exposure_s: float, scan_context: stem_controller.ScanContext, parked_probe_position: typing.Optional[Geometry.FloatPoint]) -> DataAndMetadata.DataAndMetadata:
+        frame_settings = self._get_frame_settings(readout_area, binning_shape, exposure_s, scan_context, parked_probe_position)
+        if frame_settings != self._last_frame_settings:
             self._needs_recalculation = True
-        if self.instrument.sample != self.__last_sample:
-            self._needs_recalculation = True
-        self._last_frame_settings = new_frame_settings
+            self._last_frame_settings = frame_settings
 
         if self._needs_recalculation or self.__cached_frame is None:
-            #print("recalculating frame")
+            # print("recalculating frame")
             height = readout_area.height
             width = readout_area.width
             offset_m = self.instrument.stage_position_m
+            # full_fov_nm = abs(self.instrument.GetVal("C10Control")) * self._tv_pixel_angle * self._sensor_dimensions.height * 1e9
             full_fov_nm = self.__stage_size_nm
             fov_size_nm = Geometry.FloatSize(full_fov_nm * height / self._sensor_dimensions.height, full_fov_nm * width / self._sensor_dimensions.width)
-            center_nm = Geometry.FloatPoint(full_fov_nm * (readout_area.center.y / self._sensor_dimensions.height- 0.5), full_fov_nm * (readout_area.center.x / self._sensor_dimensions.width - 0.5))
+            center_nm = Geometry.FloatPoint(
+                full_fov_nm * (readout_area.center.y / self._sensor_dimensions.height - 0.5),
+                full_fov_nm * (readout_area.center.x / self._sensor_dimensions.width - 0.5))
             size = Geometry.IntSize(height, width)
-            data = numpy.zeros((height, width), numpy.float32)
+            data: numpy.typing.NDArray[numpy.float32] = numpy.zeros((height, width), numpy.float32)
             # features will be positive values; thickness can be simulated by subtracting the features from the
             # vacuum value. the higher the vacuum value, the thinner (i.e. less contribution from features).
             thickness_param = 100
@@ -352,24 +360,18 @@ class RonchigramCameraSimulator(CameraSimulator.CameraSimulator):
                 self.instrument.sample.plot_features(data, offset_m, fov_size_nm, Geometry.FloatPoint(), center_nm, size)
                 data = thickness_param - data
             data = self._get_binned_data(data, binning_shape)
-            self.__last_sample = self.instrument.sample
 
             if not self.instrument.is_blanked:
-                probe_position = Geometry.FloatPoint(0.5, 0.5)
-                if self.instrument.probe_state == "scanning":
-                    probe_position = self.instrument.live_probe_position
-                elif self.instrument.probe_state == "parked" and parked_probe_position is not None:
-                    probe_position = parked_probe_position
-
                 scan_offset = Geometry.FloatPoint()
-                if scan_context.is_valid and probe_position is not None:
+                scan_context_fov_nm = scan_context.fov_size_nm
+                if scan_context.is_valid and frame_settings.current_probe_position is not None and scan_context_fov_nm is not None:
                     scan_offset = Geometry.FloatPoint(
-                        y=probe_position[0] * scan_context.fov_size_nm[0] - scan_context.fov_size_nm[0] / 2,
-                        x=probe_position[1] * scan_context.fov_size_nm[1] - scan_context.fov_size_nm[1] / 2)
+                        y=frame_settings.current_probe_position[0] * scan_context_fov_nm[0] - scan_context_fov_nm[0] / 2,
+                        x=frame_settings.current_probe_position[1] * scan_context_fov_nm[1] - scan_context_fov_nm[1] / 2)
                     scan_offset = scan_offset*1e-9
 
-                theta = self.__tv_pixel_angle * self._sensor_dimensions.height / 2  # half angle on camera
-                aberrations = dict()
+                theta = self._tv_pixel_angle * self._sensor_dimensions.height / 2  # half angle on camera
+                aberrations: typing.Dict[str, typing.Union[float, int]] = dict()
                 aberrations["height"] = data.shape[0]
                 aberrations["width"] = data.shape[1]
                 aberrations["theta"] = theta
@@ -401,13 +403,14 @@ class RonchigramCameraSimulator(CameraSimulator.CameraSimulator):
             self._needs_recalculation = False
 
         self.noise.poisson_level = self.__data_scale
+        assert self.__cached_frame
         return self.noise.apply(self.__cached_frame * self.__data_scale)
 
-    def get_dimensional_calibrations(self, readout_area: Geometry.IntRect, binning_shape: Geometry.IntSize):
+    def get_dimensional_calibrations(self, readout_area: typing.Optional[Geometry.IntRect], binning_shape: typing.Optional[Geometry.IntSize]) -> typing.Sequence[Calibration.Calibration]:
         height = readout_area.height if readout_area else self._sensor_dimensions[0]
         width = readout_area.width if readout_area else self._sensor_dimensions[1]
-        scale_y = self.__tv_pixel_angle
-        scale_x = self.__tv_pixel_angle
+        scale_y = self._tv_pixel_angle
+        scale_x = self._tv_pixel_angle
         offset_y = -scale_y * height * 0.5
         offset_x = -scale_x * width * 0.5
         dimensional_calibrations = [
